/*
	ProWesS, PROGS Windowing System

	automatic application installation program for ProWesS

	The progrm first starts by requesting the directory to install from.
	This directory should have a "install" subdirectory, which
	contains all the installation stuff. This should always include a file
	(also called "install") which describes the possible installation
	options and refers to the loader files which install these options.

	To allow these loader files to be written, the installation program
	defines two "Global Variables", "INSTALL-FROM" and "INSTALL-TO", which
	define the source and destination directories for installation.

	On multi-disk distributions, the loader files for all disks should be
	replicated on each other disk. This also allows possible differences at
	the start of the action to request the insertion of nother disk.

	The "install" file has the following format (very similar to the "mbutton"
	file format) :
	first line : name of program, becomes name of installation window
	second line : default installation directory (e.g. "win1_pws_").
	Each following line of the input file can contain (same as button params) :
		-path XXX : searchpath for loader file
		-name XXX : name inside item
		XXX : name of loader file which starts the program (obligatory)
	if no name is given, then the name of the loader file will be displayed
	The length of each line is limited to 160 characters.
	The default path is "$INSTALL-FROM_install".


	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 1 April 1996 (initial menu only)
	29.04.1996 continued work to do actual installation
	10.07.1996 "install_install" file is now closed !
*/

#include "io.h"
#include "mem.h"
#include "thing.h"
#include "ProWesS.h"

#define catch(x) if (err=(x)) return err; else

#define GLOBAL_NAME "Global Variables"
#define GLOBAL_GET	0x47455420	/* "GET " */
#define GLOBAL_SET	0x53455420	/* "SET " */
#define GLOBAL_DELE 0x44454c45	/* "DELE" */

#define MAX_NAME 50
#define MAX_LINE 160

static char InfoItem[]={"This is the ProWesS installation program\n"
						"Please insert the disk with the applications\n"
						"you want to install."};
static char LabelItem[]="install from : ";

typedef struct {
	char name[MAX_NAME];		/* has to be first member !! */
	char *path, *file;
	char terminate;
} Line;

static Error startinstall(PWObject object);
static Error setdest(PWObject object);
static Error install(PWObject object);

Error init()
{
	Error err;
	char work=FALSE;
	char installfrom[IO_MAXDIRECTORY];
	PWObject win, item, menu;
	Channel chid;
	char line[MAX_LINE], title[MAX_LINE];
	char **params;

	catch( PWCreate(NULL,&win,PW_TYPE_OUTLINE,
				PW_OUTLINE_SLEEP,
				PW_OUTLINE_QUIT,
				PW_OUTLINE_QUIT_KEYPRESS, 27,
				PW_OUTLINE_ACTION_DO, install,
				PW_GLOBAL_AUXILIARY, &work,
				NULL) );
	catch( PWCreate(win,NULL,PW_TYPE_INFOSTRING,
				PW_INFOSTRING_TEXT, InfoItem,
				NULL) );
	catch( PWCreate(win,&item,PW_TYPE_EDLINE,
				PW_EDLINE_MAXLENGTH, IO_MAXDIRECTORY,
				PW_EDLINE_ACTION_AFTER, install,
				PW_EDLINE_SET, "flp1_",
				NULL) );
	catch( PWCreate(win,NULL,PW_TYPE_LABEL,
				PW_POSITION_LEFT_OF, item,
				PW_SCALE_FACTOR, 0,
				PW_LABEL_TEXT, LabelItem,
				NULL) );
	
	PWActivate(win);

	PWChange(item,PW_EDLINE_GET,IO_MAXDIRECTORY,installfrom,NULL);

	PWRemove(win);
	if (!work) return ERR_OK;

	/* display actual installation window */
	catch( IOOpenPath("install_install",OPEN_READ,installfrom,&chid) );

	/* read name of program to install */
	catch( IOGetString(chid, title, MAX_LINE) );
	catch( IOGetString(chid, line, MAX_LINE) );
	THINGCall(GLOBAL_NAME,GLOBAL_SET,2,"INSTALL-FROM",installfrom);
	THINGCall(GLOBAL_NAME,GLOBAL_SET,2,"INSTALL-TO",line);

	catch( PWCreate(NULL, &win, PW_TYPE_OUTLINE,
					PW_OUTLINE_TITLE_TEXT, title,
					PW_OUTLINE_SLEEP,
					PW_OUTLINE_QUIT,
					PW_OUTLINE_QUIT_KEYPRESS, 27,
					PW_OUTLINE_ACTION_DO, startinstall,
					NULL) );
	catch( PWCreate(win, NULL, PW_TYPE_EDLINE,
					PW_EDLINE_MAXLENGTH, IO_MAXDIRECTORY,
					PW_EDLINE_ACTION_AFTER, setdest,
					PW_EDLINE_SET, line,
					PW_SCALE_FACTOR, 0,
					NULL) );
	catch( PWCreate(win, NULL, PW_TYPE_SEPARATOR, NULL) );
	catch( PWCreate(win, &menu, PW_TYPE_MENU,
					PW_MENU_VISIBLE_LINES, 8,
					PW_MENU_WINDOW_DO, TRUE,
					NULL) );
	PWChange(win,PW_GLOBAL_AUXILIARY,menu,NULL);

	while (ERR_EOF!=IOGetString(chid,line,MAX_LINE))
	{
		Line *this;

		catch( MEMAllocate(sizeof(Line),&this) );
		this->path="$INSTALL-FROM_install";

		/* parse the command line parameters */
		catch( STRBreak(line,&params) );
		for ( ; *params ; params++)
		{
			if (STRSameCI("-path",*params))
			{
				if (*++params)
					this->path=*params;
				else
					return ERR_IPAR;
			}
			else
			if (STRSameCI("-terminate",*params))
				this->terminate=TRUE;
			else
			if (STRSameCI("-name",*params))
			{
				if (*++params)
					STRCopySafe(*params,this->name,MAX_NAME);
				else
					return ERR_IPAR;
			}
			else
				this->file=*params;
		}
		if (this->file)
		{
			if (!this->name[0]) STRCopySafe(this->file,this->name,MAX_LINE);
			PWChange(menu,PW_MENU_ADD,this->name,NULL);
		}
	}
	IOClose(chid);
	PWActivate(win);

	THINGCall(GLOBAL_NAME,GLOBAL_DELE,1,"INSTALL-FROM");
	THINGCall(GLOBAL_NAME,GLOBAL_DELE,1,"INSTALL-TO");

	return ERR_OK;
}

static Error install(PWObject object)
{
	char *install;
	PWQuery(object,PW_GLOBAL_AUXILIARY,&install);
	*install=TRUE;
	return PWChange(object,PW_SYSTEM_BREAKDOWN,NULL);
}

static Error startinstall(PWObject object)
{
	PWObject menu;
	char *item, *strs[3];
	Error err;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&menu);

	for ( PWQuery(menu,PW_MENU_SELECTED_FIRST,&item) ;
		  item ;
		  PWChange(menu,PW_MENU_STATUS_CURRENT,PW_STATUS_AVAILABLE,NULL),
		  PWQuery(menu,PW_MENU_SELECTED_NEXT,&item))
	{
		Line *g=(Line *)item;

		strs[0]="-path";
		strs[1]=g->path;
		strs[2]=g->file;
		
		err=THINGExec("ProWesS loader",8,g->terminate?SYSTEM:MYSELF,
								 g->terminate?FALSE:TRUE,3,strs,0,NULL,NULL);
		if (g->terminate) JOBZap(MYSELF,ERR_OK);
		if (err)
		{
			char message[ERR_MAXLEN];

			ERRMessageGet(err,message);
			strs[0]="Unable to process";
			strs[1]=g->name;
			strs[2]=message;
			catch( THINGExec("request",8,MYSELF,FALSE,3,strs,0,NULL,NULL) );
		}
	}
	return ERR_OK;
}

static Error setdest(PWObject object)
{
	char dir[IO_MAXDIRECTORY];

	PWChange(object,PW_EDLINE_GET,IO_MAXDIRECTORY,dir,NULL);
	THINGCall(GLOBAL_NAME,GLOBAL_SET,2,"INSTALL-TO",dir);
	return ERR_OK;
}
